#!/usr/bin/env python3
"""Trajectory metric package."""

# Aggregation
from .type.aggregation import (
    AggregationTrajectoryMetric,
    AggregationTrajectoryMetricSettings,
)

__all__ = [
    "AggregationTrajectoryMetric",
    "AggregationTrajectoryMetricSettings",
]
